module alarm_fsm (
	reset, clk, alarm_on_off,
        compare_flag,
	alarm_out, alarm_anzeige
);

input reset, clk, alarm_on_off;
input compare_flag;
output alarm_out, alarm_anzeige;

`define idle		0
`define wait_state	1
`define zaehlen_on	2
`define alarm		3
`define zaehlen_off	4

reg [2:0] current_state, next_state;
reg [3:0] counter_s, counter;
reg alarm_out, alarm_anzeige;

	always @ (current_state or alarm_on_off or counter_s or compare_flag)
	begin
		counter = counter_s;

		case (current_state)
			`idle:
			begin
				alarm_out = 0;
				alarm_anzeige = 0;
		
				if (alarm_on_off == 1)  
					next_state = `zaehlen_on;
				else 
					next_state = `idle;
			end
		
			`zaehlen_on:
			begin
				alarm_out = 0;
				alarm_anzeige = 1;
	
				if (counter_s != 'b0011) 
				begin
					next_state = `zaehlen_on;
					counter = counter_s + 1;
				end
				else
				begin 
					counter =  0;
					next_state = `wait_state;
				end
			end	

			`wait_state:
			begin			
				if (compare_flag == 1) 
					next_state = `alarm;
				else if (alarm_on_off == 0) 
					next_state = `zaehlen_off;
				else
					next_state = `wait_state;
			
				alarm_out = 0;
				alarm_anzeige = 1;
			end	
					
			`zaehlen_off:
			begin
				alarm_out = 0;
				alarm_anzeige = 0;

				if (counter_s != 'b0011) 
				begin
					next_state = `zaehlen_off;
					counter = counter_s + 1;
				end
				else 
				begin
					counter =  0;
					next_state = `idle;
				end	
			end

			`alarm:
			begin
				alarm_out = 1;
				alarm_anzeige = 1;
			
				if (alarm_on_off == 0) 
				begin
 					if (counter_s != 'b0011) 
					begin
						next_state = `zaehlen_off;
						counter = counter_s + 1;
					end
					else 
					begin
						counter =  0;
						next_state = `idle;
					end
				end
				else 
					next_state = `alarm;				
			end
			
			default:
			begin
				alarm_out = 0;
				alarm_anzeige = 0;
				next_state = `idle;
				counter =  0;
			end
		endcase
	end

	always @ (posedge clk or posedge reset)
	begin
		if (reset == 1)
		begin	
			current_state = `idle;
			counter_s = 0;
		end
		else 
		begin
			current_state = next_state;
			counter_s = counter;
		end
	end
endmodule
