module time_block2 (
	reset, clk, set_time_flag, set_hour, set_min,
	set_alarm_flag,
        compare_flag, s,
	m_1, h_1,
	m_10,
	h_10
);


input reset, clk, set_time_flag, set_hour, set_min;
input set_alarm_flag;
output compare_flag, s;
output [3:0]m_1, h_1;
output [2:0]m_10;
output [1:0]h_10;

`define idle			0
`define set_time_state		1
`define set_alarm_state		2
`define work			3

reg [1:0] current_state, next_state;

reg [3:0] hour_1_s, hour_1, alarm_hour_1, alarm_hour_1_s;
reg [1:0] hour_10_s, hour_10, alarm_hour_10, alarm_hour_10_s;

reg [3:0] min_1_s, min_1, alarm_min_1, alarm_min_1_s;
reg [2:0] min_10_s, min_10, alarm_min_10, alarm_min_10_s;
reg [2:0] sec_10_s, sec_10;
reg [3:0] sec_1_s, sec_1;
reg compare_flag, s;
reg [3:0] m_1, h_1;
reg [2:0] m_10;
reg [1:0] h_10;

	always @ (current_state or sec_1_s or sec_10_s or min_1_s or min_10_s or hour_1_s or 
		  hour_10_s or set_alarm_flag or set_time_flag or set_hour or set_min or
		  alarm_hour_1_s or alarm_hour_10_s or alarm_min_1_s or alarm_min_10_s)
	begin
		hour_1 = hour_1_s; hour_10 = hour_10_s; min_1 = min_1_s; min_10 = min_10_s; sec_1 = sec_1_s; sec_10 = sec_10_s;
		alarm_hour_1 = alarm_hour_1_s; alarm_min_1 = alarm_min_1_s; 
		alarm_hour_10 = alarm_hour_10_s; alarm_min_10 = alarm_min_10_s; 
		
		case (current_state)
	
			`idle:
			begin
				hour_1 = 0; hour_10 = 0; min_1 = 0; min_10 = 0; sec_1 = 0; sec_10 = 0;
				
				next_state = `work;
			end
			
			`work:
			begin
				sec_1 = sec_1_s + 1;
				min_1 = min_1_s;
				min_10 = min_10_s;
				hour_1 = hour_1_s;
				hour_10 = hour_10_s;
				
				if (set_time_flag == 1)
					next_state = `set_time_state;
				else if (set_alarm_flag == 1)
					next_state = `set_alarm_state;
				else
					next_state = `work;
				
				if (sec_1_s == 'b1001)
				begin
					sec_1 = 0;
					sec_10 = sec_10_s + 1;
				end

				if (sec_10_s == 'b101 && sec_1_s == 'b1001)
				begin
					sec_10 = 0;
					min_1 = min_1_s + 1;
				end

				if (min_1_s == 'b1001 && sec_10_s == 'b101 && sec_1_s == 'b1001) 
				begin
					min_1 = 0;
					min_10 = min_10_s + 1;
				end
				
				if (min_10_s == 'b101 && min_1_s == 'b1001 && sec_10_s == 'b101 && sec_1_s == 'b1001)
				begin
					min_10 = 0;
					hour_1 = hour_1_s + 1;
				end
				
				if (hour_1_s == 'b11 && min_10_s == 'b101 && min_1_s == 'b1001 && sec_10_s == 'b101 && sec_1_s == 'b1001)
				begin
					hour_1 = 0;
					hour_10 = hour_10_s + "01";
				end

				if (hour_10_s == 'b10 && hour_1_s <= 'b11 && min_10_s == 'b101 && min_1_s == 'b1001 && sec_10_s == 'b101 && sec_1_s == 'b1001)
					hour_10 = 0;
			end					
				

			`set_time_state:
			begin
				if (set_time_flag == 1 && set_hour == 1)
				begin 
					hour_1 = hour_1_s + 1;
		
					if (hour_1_s == 'b1001)
					begin 
						hour_1 = 0;
						hour_10 = hour_10_s + 1;
					end
					else if (hour_10_s == 'b10 && hour_1_s == 'b0011)
					begin 
						hour_10 = 0;
						hour_1 = 0;
					end
				
					next_state = `set_time_state;
				end
				if (set_time_flag == 1 && set_min == 1)
				begin 
					min_1 = min_1_s + 1;
				
					if (min_1_s == 'b1001)
					begin 
						min_1 = 0;
						min_10 = min_10_s + 1;
					end
				
					if (min_10_s == 'b101 && min_1_s == 'b1001) 
						min_10 = 0;
				
					next_state = `set_time_state;
				end
				else
					next_state = `work;
			
				sec_1 = 0;			
			end
			
			`set_alarm_state:
			begin
				if (set_alarm_flag == 1 && set_hour == 1)
				begin 
					alarm_hour_1 = alarm_hour_1_s + 1;
				
					if (alarm_hour_1_s == 'b1001) 
					begin 
						alarm_hour_1 = 0;
						alarm_hour_10 = alarm_hour_10_s + 1;
					end
				
					if (alarm_hour_10_s == 'b10 && alarm_hour_1_s == 'b0011)
					begin 
						alarm_hour_10 = 0;
						alarm_hour_1 = 0;
					end
				
					next_state = `set_alarm_state;
				end
				if (set_alarm_flag == 1 && set_min == 1)
				begin 
					alarm_min_1 = alarm_min_1_s + 1;
				
					if (alarm_min_1_s == 'b1001) 
					begin 
						alarm_min_1 = 0;
						alarm_min_10 = alarm_min_10_s + 1;
					end
					
					if (alarm_min_10_s == 'b101 && alarm_min_1_s == 'b1001)
					begin 
						alarm_min_10 = 0;
						alarm_min_1  = 0;
					end
				
					next_state = `set_alarm_state;
				end
				else
					next_state = `work;
			
				sec_1 = 0;			
			end
			
			default:
			begin
				hour_1 = hour_1_s; hour_10 = hour_10_s; min_1 = min_1_s; min_10 = min_10_s; sec_1 = sec_1_s; sec_10 = sec_10_s;
				alarm_hour_1 = alarm_hour_1_s; alarm_min_1 = alarm_min_1_s; 
				alarm_hour_10 = alarm_hour_10_s; alarm_min_10 = alarm_min_10_s; 
				compare_flag = 0;
				next_state = `idle;
			end
		endcase
	end

	always @ (hour_10_s or hour_1_s or min_10_s or min_1_s or alarm_hour_10_s or alarm_hour_1_s or alarm_min_10_s or alarm_min_1_s)
	begin
		if ((set_alarm_flag == 0) && (alarm_hour_10_s == hour_10_s) && (alarm_min_10_s == min_10_s) && (alarm_hour_1_s == hour_1_s) && (alarm_min_1_s == min_1_s))
			compare_flag = 1;
		else 
			compare_flag = 0;
	end




	always @ (posedge clk or posedge reset)
	begin
		if (reset == 1) 
		begin
			hour_1_s = 0; alarm_hour_1_s = 0; min_1_s = 0; alarm_min_1_s = 0; sec_1_s = 0;
			hour_10_s = 0; alarm_hour_10_s = 0; min_10_s = 0; alarm_min_10_s = 0;
			current_state = `idle;
		end
		else
		begin
			hour_1_s = hour_1; min_1_s = min_1; sec_1_s = sec_1; hour_10_s = hour_10; min_10_s = min_10; sec_10_s = sec_10;
			alarm_hour_1_s = alarm_hour_1; alarm_min_1_s = alarm_min_1;
			alarm_hour_10_s = alarm_hour_10; alarm_min_10_s = alarm_min_10;
			current_state = next_state;
		end
	end


	always @ (set_alarm_flag or alarm_hour_1_s or alarm_hour_10_s or alarm_min_1_s or alarm_min_10_s or clk or hour_1_s or hour_10_s or min_1_s or min_10_s)
	begin
		if (set_alarm_flag == 1) 
		begin
			h_1 = alarm_hour_1_s;
			h_10 = alarm_hour_10_s;
			m_1 = alarm_min_1_s;
			m_10 = alarm_min_10_s;
			s = clk;
		end
		else
		begin
			h_1 = hour_1_s;
			h_10 = hour_10_s;
			m_1 = min_1_s;
			m_10 = min_10_s;
			s = clk;
		end
	end

endmodule

